/****************************************************************************************/
/* Web.ino                                                                              */
/* Collection of subroutines to display and process webpages                            */
/* SUB_DEFINE_ROUTES:       Subroutine to define the routes for ESPAsyncWebserver       */
/*                          to be once during setup                                     */
/* SUB_MESSAGE_ERROR:       Subroutine to be called if message error is detected        */   
/* SUB_WEB_PROCESSOR_MAIN:  Subroutine to replace placeholder in Websites               */
/*                          with calculated values                                      */  
/* SUB_WEB_PROCESSOR_SUBM1: Subroutine to replace placeholder in website submenu 1      */
/*                          with calculated values                                      */
/* SUB_WEB_PROCESSOR_SUBM2: Subroutine to replace placeholder in website submenu 2      */
/*                          with calculated values                                      */       
/* SUB_WEB_PROCESSOR_SUBM3: Subroutine to replace placeholder in website submenu 3      */
/*                          with calculated values                                      */       
/****************************************************************************************/


/****************************************************************************************/
/* Subroutine to define the routes for ESPAsyncWebserver                                */
/****************************************************************************************/
void SUB_DEFINE_ROUTES (void)
{
  /********************* Routes for loading CSS style sheet  ***************************/
  server.on("/style.css", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    /* Answer request by opening the style sheet */
    DISPLAY_LOCK = true;                          /* Set display lock true */                            
    request->send(SPIFFS, "/Style.css", "text/css");
    DISPLAY_LOCK = false;                         /* Set display lock false */

  });
    
  /**************************** Routes for main page **********************************/
  server.on("/", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();               
      Serial.println ("Client connected: Display main page...");
    #endif  
    DISPLAY_LOCK = true;                          /* Set display lock true */
    /* Answer request with displaying of main web page */ 
    request->send(SPIFFS, "/Main.html", String(), false, SUB_WEB_PROCESSOR_MAIN);
    DISPLAY_LOCK = false;                         /* Set display lock false */ 
  });

  server.on("/main", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();               
      Serial.println ("Client connected: Display main page...");
    #endif  
    DISPLAY_LOCK = true;                          /* Set display lock true */
    /* Answer request with displaying of main web page */
    request->send(SPIFFS, "/Main.html", String(), false, SUB_WEB_PROCESSOR_MAIN);
    DISPLAY_LOCK = false;                         /* Set display lock false */ 
  });
  
  /* Main menu: Button for device on/off pressed */
  server.on("/device", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();      
      Serial.println ("Main menu: Button for device on/off pressed...");
    #endif  
    DISPLAY_LOCK = true;                          /* Set display lock true */
    /* Answer request by refreshing main page */
    request->send(SPIFFS, "/Main.html", String(), false, SUB_WEB_PROCESSOR_MAIN);
    DISPLAY_LOCK = false;                         /* Set display lock false */
     
    if (POWER_OFF == true)                        /* Device in power off state ? */
    {
      SELECT = 129;                               /* Set selct code to switch device on */  
      POWER_WEB_OFF = false;                      /* Set power web off false */                                
    }
    else                                          /* Device in power on state ? */
    {
      SELECT = 128;                               /* Set selct code to switch device off */ 
      POWER_WEB_OFF = true;                       /* Set power web off true */              
    }
  });

  /* Main menu: Refresh button pressed */
  server.on("/refreshm", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Main menu: Button for refresh of page pressed...");
    #endif 
    DISPLAY_LOCK = true;                          /* Set display lock true */ 
    /* Answer request by refreshing main page */
    request->send(SPIFFS, "/Main.html", String(), false, SUB_WEB_PROCESSOR_MAIN);
    DISPLAY_LOCK = false;                         /* Set display lock false */ 
  });

  /************************** Routes for submenu page 1 *******************************/
   server.on("/submenu1", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();               
      Serial.println ("Client connected: Display submenu 1 page...");
    #endif 
    DISPLAY_LOCK = true;                          /* Set display lock true */ 
    /* Answer request with displaying of submain page 1 web page */
    request->send(SPIFFS, "/Submenu1.html", String(), false, SUB_WEB_PROCESSOR_SUBM1);
    DISPLAY_LOCK = false;                         /* Set display lock false */
  }); 

   /* Submenu 1: Button for saving of the values pressed */
  server.on("/store1", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */
    int TEMP_SELECT;                              /* Temporarry integer for select value */
        
    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Submenu 1: Button for saving of values pressed...");
    #endif 
    DISPLAY_LOCK = true;                          /* Set display lock true */
    if (request->hasParam("SUB1COLOR"))           /* Variable single color available? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB1COLOR")->value();
      TEMP_SELECT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Color set to: ");
        switch (TEMP_SELECT)
        {
          case 0:
            Serial.println ("---");
            break;
          case 1:
            Serial.println ("Cold white");
            break;
          case 2:
            Serial.println ("Warm white");
            break;
          case 3:
            Serial.println ("Yellow");
            break;
          case 4:
            Serial.println ("Orange");
            break;
          case 5:
            Serial.println ("Red");
            break;
          case 6:
            Serial.println ("Dark purple");
            break; 
          case 7:
            Serial.println ("Purple");
            break;
          case 8:
            Serial.println ("Bright purple");
            break; 
          case 9:
            Serial.println ("Blue");
            break; 
          case 10:
            Serial.println ("Bright blue");
            break; 
          case 11:
            Serial.println ("Cyan");
            break; 
          case 12:
            Serial.println ("Bright green");
            break; 
          case 13:
            Serial.println ("Green");
            break;
        }                        
      #endif 
    } 

    if (request->hasParam("SUB1DIM"))             /* Variable single color brightness available? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB1DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Brightness for color set to: ");
        switch (TEMP_INT)
        {
          case 0:
            Serial.println ("100%");
            break; 
          case 1:
            Serial.println ("85%");
            break; 
          case 2:
            Serial.println ("70%");
            break; 
          case 3:
            Serial.println ("60%");
            break; 
          case 4:
            Serial.println ("50%");
            break; 
          case 5:
            Serial.println ("40%");
            break; 
          case 6:
            Serial.println ("30%");
            break;                                     
          case 7:
            Serial.println ("20%");
            break; 
          case 8:
            Serial.println ("10%");
            break; 
        }                        
      #endif 
      COLOR_DIM_STEP = TEMP_INT;                  /* Save diming factor for fixed color */
      EEPROM.write(1,COLOR_DIM_STEP);             /* Save diming factor for fixed color in FLASH  */
      EEPROM.commit();                            /* Store values permanently in FLASH */         
    }    
    /* Answer request with displaying of submain page 2 web page */
    request->send(SPIFFS, "/Submenu1.html", String(), false, SUB_WEB_PROCESSOR_SUBM1);
    DISPLAY_LOCK = false;                         /* Set display lock false */
    /* In range? */
    if ((TEMP_SELECT >=1)&& (TEMP_SELECT <= 13))  
    {
      SELECT = TEMP_SELECT;    
      ACT_PROGRAM = SELECT;                       /* Save in actual program */         
    } 
  }); 
 
  /************************** Routes for submenu page 2 *******************************/
   server.on("/submenu2", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();               
      Serial.println ("Client connected: Display submenu 2 page...");
    #endif  
    DISPLAY_LOCK = true;                          /* Set display lock true */
    /* Answer request with displaying of submain page 2 web page */
    request->send(SPIFFS, "/Submenu2.html", String(), false, SUB_WEB_PROCESSOR_SUBM2);
    DISPLAY_LOCK = false;                         /* Set display lock false */
  });   

  /* Submenu 2: Button for saving of the values pressed */
  server.on("/store2", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */
    int TEMP_PROG;                                /* Temporarry integer for program */
        
    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Submenu 2: Button for saving of values pressed...");
    #endif 
    DISPLAY_LOCK = true;                          /* Set display lock true */
    if (request->hasParam("SUB2PROG"))            /* Variable program available? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB2PROG")->value();
      TEMP_PROG = TEMP_BUFFER.toInt();            /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Program set to: ");
        switch (TEMP_PROG)
        {          
          case 0:
            Serial.println ("---");
            break;           
          case 1:
            Serial.println ("Color change 1");
            break;
          case 2:
            Serial.println ("Color change 2");
            break;
          case 3:
            Serial.println ("Color change 3");
            break;
          case 4:
            Serial.println ("Color change 4");
            break;
          case 5:
            Serial.println ("Color change 5");
            break;
          case 6:
            Serial.println ("Steps");
            break; 
          case 7:
            Serial.println ("Color flash 1");
            break;
          case 8:
            Serial.println ("Color flash 2");
            break; 
          case 9:
            Serial.println ("Thunderstorm");
            break; 
          case 10:
            Serial.println ("Gewitter");
            break; 
        }                        
      #endif 
    } 

    if (request->hasParam("SUB2DIM"))             /* Variable program brightness available? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB2DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Brightness for program set to: ");
        switch (TEMP_INT)
        {
        
          case 0:
            Serial.println ("100%");
            break; 
          case 1:
            Serial.println ("75%");
            break; 
          case 2:
            Serial.println ("50%");
            break; 
          case 3:
            Serial.println ("25%");
            break; 
        }                        
      #endif 
      PROG_DIM_STEP = TEMP_INT;                   /* Save diming factor for program */
      EEPROM.write(2,PROG_DIM_STEP);              /* Save diming factor for program in FLASH  */
      EEPROM.commit();                            /* Store values permanently in FLASH */         
    }  

    if (request->hasParam("SUB2SPEED"))           /* Variable program speed available? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB2SPEED")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Speed for program set to: ");
        switch (TEMP_INT)
        {
          case 0:
            Serial.println ("100%");
            break; 
          case 1:
            Serial.println ("90%");
            break; 
          case 2:
            Serial.println ("80%");
            break; 
          case 3:
            Serial.println ("70%");
            break; 
          case 4:
            Serial.println ("60%");
            break;        
          case 5:
            Serial.println ("50%");
            break;    
          case 6:
            Serial.println ("40%");
            break; 
          case 7:
            Serial.println ("30%");
            break; 
          case 8:
            Serial.println ("20%");
            break;    
          case 9:
            Serial.println ("10%");
            break;                                    
        }                        
      #endif 
      PROG_SPEED = TEMP_INT;                      /* Save speed factor for program */
      EEPROM.write(3,PROG_SPEED);                 /* Save diming factor for program in FLASH  */
      EEPROM.commit();                            /* Store values permanently in FLASH */         
    }

    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu2.html", String(), false, SUB_WEB_PROCESSOR_SUBM2);
    DISPLAY_LOCK = false;                         /* Set display lock false */
    if ((TEMP_PROG >=1)&& (TEMP_PROG <= 10))      /* In range? */  
    {
      SELECT = TEMP_PROG + 19; 
      ACT_PROGRAM = SELECT;                       /* Save in actual program */           
    }
  }); 

  /* Submenu 2: Button for pause pressed */
  server.on("/pause", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Submenu 2: Button for pause of program pressed...");
    #endif     
    DISPLAY_LOCK = true;                          /* Set display lock true */    
    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu2.html", String(), false, SUB_WEB_PROCESSOR_SUBM2);
    DISPLAY_LOCK = false;                         /* Set display lock false */
    SELECT = 130;                                 /* Send code for pause */     
  });

  /* Submenu 2: Button for play pressed */
  server.on("/play", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Submenu 2: Button for play of program pressed...");
    #endif         
    DISPLAY_LOCK = true;                          /* Set display lock true */
    /* Answer request with displaying of submain page 2 web page */
    request->send(SPIFFS, "/Submenu2.html", String(), false, SUB_WEB_PROCESSOR_SUBM2);
    DISPLAY_LOCK = false;                         /* Set display lock false */
    SELECT = 131;                                 /* Send code for play */ 
  });

  /************************** Routes for submenu page 3 *******************************/
   server.on("/submenu3", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();               
      Serial.println ("Client connected: Display submenu 3 page...");
    #endif  
    SUB_MENU_FLAG = false;                        /* Set menu flag false = Display zero */
    DISPLAY_LOCK = true;                          /* Set display lock true */
    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
    DISPLAY_LOCK = false;                         /* Set display lock false */
  }); 

  /* Submenu 3: Button for saving of the values pressed */
  server.on("/store3", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */
    int TEMP_STORE;                               /* Temporarry integer for storage position */
        
    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Submenu 3: Button for saving of values pressed...");
    #endif 
    DISPLAY_LOCK = true;                          /* Set display lock true */
    if (request->hasParam("SUB3STORE"))           /* Variable program available? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3STORE")->value();
      TEMP_STORE = TEMP_BUFFER.toInt();           /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Program set to: ");
        switch (TEMP_STORE)
        {          
          case 0:
            Serial.println ("---");
            break;
          case 1:
            Serial.println ("Storage 1");
            break;
          case 2:
            Serial.println ("Storage 2");
            break;
          case 3:
            Serial.println ("Storage 3");
            break;
          case 4:
            Serial.println ("Storage 4");
            break;
          case 5:
            Serial.println ("Storage 5");
            break;
        }                        
      #endif 
      SUB_MENU_FLAG = false;                      /* Set menu flag false = Display zero */
    } 

    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
    DISPLAY_LOCK = false;                         /* Set display lock false */
    if ((TEMP_STORE >=1)&& (TEMP_STORE <= 5))     /* In range? */  
    {
      SELECT = TEMP_STORE + 14; 
      ACT_PROGRAM = SELECT;                       /* Save in actual program */           
    }
  }); 

  /* Submenu 3: Button for recalling storage 1 pressed */
  server.on("/rcl1", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Submenu 3: Button for recalling storage position 1 pressed...");
    #endif  
    DISPLAY_LOCK = true;                          /* Set display lock true */   
    STORE_RED_COLOR   = EEPROM.read (10);         /* Red color value of storage position 1 */
    STORE_GREEN_COLOR = EEPROM.read (11);         /* Green color value of storage position 1 */
    STORE_BLUE_COLOR  = EEPROM.read (12);         /* Blue color value of storage position 1 */
    STORE_DIM_VALUE   = EEPROM.read (13);         /* Dimming value of storage position 1 */ 
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */

    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
    DISPLAY_LOCK = false;                         /* Set display lock false */
    SELECT = 15;                                   /* Set program 1 */
  });

  /* Submenu 3: Button for recalling storage 2 pressed */
  server.on("/rcl2", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();      
      Serial.println ("Submenu 3: Button for recalling storage position 2 pressed...");
    #endif       
    DISPLAY_LOCK = true;                          /* Set display lock true */
    STORE_RED_COLOR   = EEPROM.read (20);         /* Red color value of storage position 2 */
    STORE_GREEN_COLOR = EEPROM.read (21);         /* Green color value of storage position 2 */
    STORE_BLUE_COLOR  = EEPROM.read (22);         /* Blue color value of storage position 2 */
    STORE_DIM_VALUE   = EEPROM.read (23);         /* Dimming value of storage position 2 */   
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */

    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
    DISPLAY_LOCK = false;                         /* Set display lock false */
    SELECT = 16;                                  /* Set program 2 */
  });

  /* Submenu 3: Button for recalling storage 3 pressed */
  server.on("/rcl3", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Submenu 3: Button for recalling storage position 3 pressed...");
    #endif   
    DISPLAY_LOCK = true;                          /* Set display lock true */
    STORE_RED_COLOR   = EEPROM.read (30);         /* Red color value of storage position 3 */
    STORE_GREEN_COLOR = EEPROM.read (31);         /* Green color value of storage position 3 */
    STORE_BLUE_COLOR  = EEPROM.read (32);         /* Blue color value of storage position 3 */
    STORE_DIM_VALUE   = EEPROM.read (33);         /* Dimming value of storage position 3 */ 
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */

    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
    DISPLAY_LOCK = false;                         /* Set display lock false */
    SELECT = 17;                                  /* Set program 3 */      
  });

  /* Submenu 3: Button for recalling storage 4 pressed */
  server.on("/rcl4", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Submenu 3: Button for recalling storage position 4 pressed...");
    #endif 
    DISPLAY_LOCK = true;                          /* Set display lock true */      
    STORE_RED_COLOR   = EEPROM.read (40);         /* Red color value of storage position 4 */
    STORE_GREEN_COLOR = EEPROM.read (41);         /* Green color value of storage position 4 */
    STORE_BLUE_COLOR  = EEPROM.read (42);         /* Blue color value of storage position 4 */
    STORE_DIM_VALUE   = EEPROM.read (43);         /* Dimming value of storage position 4 */   
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */    

    /* Answer request with displaying of submain page 4 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
    DISPLAY_LOCK = false;                         /* Set display lock false */
    SELECT = 18;                                  /* Set program 4 */    
  });

  /* Submenu 3: Button for recalling storage 5 pressed */
  server.on("/rcl5", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Submenu 3: Button for recalling storage position 5 pressed...");
    #endif     
    DISPLAY_LOCK = true;                          /* Set display lock true */  
    STORE_RED_COLOR   = EEPROM.read (50);         /* Red color value of storage position 5 */
    STORE_GREEN_COLOR = EEPROM.read (51);         /* Green color value of storage position 5 */
    STORE_BLUE_COLOR  = EEPROM.read (52);         /* Blue color value of storage position 5 */
    STORE_DIM_VALUE   = EEPROM.read (53);         /* Dimming value of storage position 5 */   
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */    

    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
    DISPLAY_LOCK = false;                         /* Set display lock false */
    SELECT = 19;                                  /* Set program 5 */    
  });

  /* Submenu 3: Button for storing storage 1 pressed */
  server.on("/sto1", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */

    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Submenu 3: Button for writing storage position 1 pressed...");
    #endif    
    DISPLAY_LOCK = true;                          /* Set display lock true */    
    if (request->hasParam("SUB3RED"))             /* Variable red value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3RED")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 1 red value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (10, TEMP_INT);                /* Write red value to storage position 1 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB3GREEN"))           /* Variable green value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3GREEN")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 1 green value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (11, TEMP_INT);                /* Write green value to storage position 1 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB3BLUE"))            /* Variable blue value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3BLUE")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 1 blue value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (12, TEMP_INT);                /* Write blue value to storage position 1 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }
        
    if (request->hasParam("SUB3DIM"))             /* Variable dim value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 1 dimming value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (13, TEMP_INT);                /* Write dimming value to storage position 1 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */      

    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
    DISPLAY_LOCK = false;                         /* Set display lock false */
  });

  /* Submenu 3: Button for storing storage 2 pressed */
  server.on("/sto2", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */
    
    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Submenu 3: Button for writing storage position 2 pressed...");
    #endif
    DISPLAY_LOCK = true;                          /* Set display lock true */          
    if (request->hasParam("SUB3RED"))             /* Variable red value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3RED")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 2 red value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (20, TEMP_INT);                /* Write red value to storage position 2 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB3GREEN"))           /* Variable green value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3GREEN")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 2 green value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (21, TEMP_INT);                /* Write green value to storage position 2 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB3BLUE"))            /* Variable blue value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3BLUE")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 2 blue value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (22, TEMP_INT);                /* Write blue value to storage position 2 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }
        
    if (request->hasParam("SUB3DIM"))             /* Variable dim value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 2 dimming value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (23, TEMP_INT);                /* Write dimming value to storage position 2 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }      
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */ 
    
    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
    DISPLAY_LOCK = false;                         /* Set display lock false */
  });

  /* Submenu 3: Button for storing storage 3 pressed */
  server.on("/sto3", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */

    #ifdef _DEBUG_
      Serial.println ();         
      Serial.println ("Submenu 3: Button for writing storage position 3 pressed...");
    #endif    
    DISPLAY_LOCK = true;                          /* Set display lock true */      
    if (request->hasParam("SUB3RED"))             /* Variable red value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3RED")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 3 red value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (30, TEMP_INT);                /* Write red value to storage position 3 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB3GREEN"))           /* Variable green value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3GREEN")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 3 green value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (31, TEMP_INT);                /* Write green value to storage position 3 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB3BLUE"))            /* Variable blue value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3BLUE")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 3 blue value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (32, TEMP_INT);                /* Write blue value to storage position 3 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }
        
    if (request->hasParam("SUB3DIM"))             /* Variable dim value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 3 dimming value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (33, TEMP_INT);                /* Write dimming value to storage position 3 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */           

    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
    DISPLAY_LOCK = false;                         /* Set display lock false */
  });

  /* Submenu 3: Button for storing storage 4 pressed */
  server.on("/sto4", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */

    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Submenu 3: Button for writing storage position 4 pressed...");
    #endif      
    DISPLAY_LOCK = true;                          /* Set display lock true */
    if (request->hasParam("SUB3RED"))             /* Variable red value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3RED")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 4 red value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (40, TEMP_INT);                /* Write red value to storage position 4 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB3GREEN"))           /* Variable green value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3GREEN")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 4 green value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (41, TEMP_INT);                /* Write green value to storage position 4 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB3BLUE"))            /* Variable blue value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3BLUE")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 4 blue value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (42, TEMP_INT);                /* Write blue value to storage position 4 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }
        
    if (request->hasParam("SUB3DIM"))             /* Variable dim value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 4 dimming value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (43, TEMP_INT);                /* Write dimming value to storage position 4 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */          

    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
    DISPLAY_LOCK = false;                         /* Set display lock false */
  });


  /* Submenu 3: Button for storing storage 5 pressed */
  server.on("/sto5", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */
    
    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Submenu 3: Button for writing storage position 5 pressed...");
    #endif     
    DISPLAY_LOCK = true;                          /* Set display lock true */     
    if (request->hasParam("SUB3RED"))             /* Variable red value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3RED")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 5 red value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (50, TEMP_INT);                /* Write red value to storage position 5 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB3GREEN"))           /* Variable green value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3GREEN")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 5 green value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (51, TEMP_INT);                /* Write green value to storage position 5 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    } 
        
    if (request->hasParam("SUB3BLUE"))            /* Variable blue value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3BLUE")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 5 blue value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (52, TEMP_INT);                /* Write blue value to storage position 5 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }
        
    if (request->hasParam("SUB3DIM"))             /* Variable dim value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Storage 5 dimming value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      EEPROM.write (53, TEMP_INT);                /* Write dimming value to storage position 5 into FLASH */
      EEPROM.commit();                            /* Store values permanently in FLASH */ 
    }      
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */ 
    
    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
    DISPLAY_LOCK = false;                         /* Set display lock false */
  });

  /* Submenu 3: Button for test pressed */
  server.on("/test", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    int TEMP_INT;                                 /* Temporarry integer for extraction */

    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Submenu 3: Button for testing of colors pressed...");
    #endif 
    DISPLAY_LOCK = true;                          /* Set display lock true */         
    if (request->hasParam("SUB3RED"))             /* Variable red value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3RED")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Test red value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      STORE_RED_COLOR = TEMP_INT;                 /* Save red value */ 
    }  
        
    if (request->hasParam("SUB3GREEN"))           /* Variable green value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3GREEN")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Test green value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      STORE_GREEN_COLOR = TEMP_INT;               /* Save green value */ 
    } 
        
    if (request->hasParam("SUB3BLUE"))            /* Variable blue value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3BLUE")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Test1 blue value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      STORE_BLUE_COLOR = TEMP_INT;                /* Save blue value */
    }
        
    if (request->hasParam("SUB3DIM"))             /* Variable dim value ? */
    {
      /* Save parameter into the receive message string */
      TEMP_BUFFER = request->getParam("SUB3DIM")->value();
      TEMP_INT = TEMP_BUFFER.toInt();             /* Convert into integer value and save variable */ 
      #ifdef _DEBUG_
        Serial.print ("  Test dimming value: ");
        Serial.println (TEMP_INT);                       
      #endif 
      STORE_DIM_VALUE = TEMP_INT;                 /* Save blue value */
    }  
    SUB_MENU_FLAG = true;                         /* Set menu flag false = Display zero */ 

    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
    DISPLAY_LOCK = false;                         /* Set display lock false */
    TEST_COLOR = true;                            /* Set test color flag true */    
  });

  /* Submenu 3: Button for restore pressed */
  server.on("/restore", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Submenu 3: Button for restoring storage position pressed...");
    #endif  
    DISPLAY_LOCK = true;                          /* Set display lock true */        
    /* Answer request with restore page */
    request->send(SPIFFS, "/Restore.html");
    DISPLAY_LOCK = false;                         /* Set display lock false */
  });

  /* Restore menu: Button for cancel pressed */
  server.on("/rcancel", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Restore menu: Cancel button pressed...");
    #endif 
    DISPLAY_LOCK = true;                          /* Set display lock true */         
    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
    DISPLAY_LOCK = false;                         /* Set display lock false */
  });
  
  /* Restore menu: Button for save pressed */
  server.on("/rstore", HTTP_GET, [](AsyncWebServerRequest *request)
  {
    #ifdef _DEBUG_
      Serial.println ();          
      Serial.println ("Restore menu: Store button pressed...");
    #endif 
    DISPLAY_LOCK = true;                          /* Set display lock true */       
    /* Answer request with displaying of submain page 3 web page */
    request->send(SPIFFS, "/Submenu3.html", String(), false, SUB_WEB_PROCESSOR_SUBM3);
    DISPLAY_LOCK = false;                         /* Set display lock false */
    SELECT = 143;                                 /* Set select to 143 --> Set default to storage positions */     
  });

  /*********************** Errorhandler on undefined routes ***************************/
  server.onNotFound(SUB_MESSAGE_ERROR);
}


/****************************************************************************************/
/* Subroutine to be called if message error is detected                                 */
/****************************************************************************************/
void SUB_MESSAGE_ERROR(AsyncWebServerRequest *request) 
{
    #ifdef _DEBUG_
      Serial.println ();               
      Serial.println("New client request...");
      Serial.println ("  Message error!!!");
    #endif  
    request->send(404, "text/plain", "Message error");
}

/****************************************************************************************/
/* Subroutine to replace placeholder in website main with calculated values             */
/****************************************************************************************/
String SUB_WEB_PROCESSOR_MAIN (const String& REPLACE_TARGET)
{
  byte TEMP_BYTE;                                 /* Temporary byte for handling of strings */
    
  if (REPLACE_TARGET == "MAINTEXT1")              /* Actual status on/off */
  {
    if (POWER_WEB_OFF == true)                    /* Power off status for web is true ? */
    {
      TEMP_BUFFER = "Off";                        /* Set temp buffer to off */
    }
    else                                          /* Power off status is false? */
    {
      TEMP_BUFFER = "On";                         /* Set temp buffer to on */
    }    
    return (TEMP_BUFFER);                         /* Return message buffer */
  }   
  if (REPLACE_TARGET == "MAINTEXT2")              /* Actual program */
  {
    TEMP_BUFFER = "-";                            /* Predefine temp buffer */
    /* Color selected */
    if ((ACT_PROGRAM >= 1) && (ACT_PROGRAM <= 13))
    {
      TEMP_BUFFER = NR_COLOR[ACT_PROGRAM-1];
    }
    /* Storage selected */
    if ((ACT_PROGRAM >= 15) && (ACT_PROGRAM <= 19))
    {
      TEMP_BUFFER = NR_STORAGE[ACT_PROGRAM-15]; 
    }
    /* Program selected */
    if ((ACT_PROGRAM >= 20) && (ACT_PROGRAM <= 29))
    {
      TEMP_BUFFER = NR_PROGRAM[ACT_PROGRAM-20];  
    }
    return (TEMP_BUFFER);                         /* Return message buffer */
  }

  if (REPLACE_TARGET == "MAINTEXT3")              /* Actual color */
  {
    TEMP_BUFFER = "-";                            /* Predefine temp buffer */
    /* Color selected */
    if ((ACT_PROGRAM >= 1) && (ACT_PROGRAM <= 13))
    {
      TEMP_BUFFER = "Color";                      /* Set temp buffer to color */ 
    }
    /* Storage selected */
    if ((ACT_PROGRAM >= 15) && (ACT_PROGRAM <= 19))
    {
      TEMP_BUFFER = "Storage";                    /* Set temp buffer to store */ 
    }
    /* Program selected */
    if ((ACT_PROGRAM >= 20) && (ACT_PROGRAM <= 29))
    {
      TEMP_BUFFER = "Program";                    /* Set temp buffer to program */ 
    }
    return (TEMP_BUFFER);                         /* Return message buffer */      
  }
  
  if (REPLACE_TARGET == "MAINTEXT4")              /* Actual dimming color */
  {
    TEMP_BYTE = COLOR_DIM_STEP;                   /* Read actual color dimming value */
    TEMP_BUFFER = DIM_COLOR[TEMP_BYTE];
    return (TEMP_BUFFER);                         /* Return message buffer */ 
  }

  if (REPLACE_TARGET == "MAINTEXT5")              /* Actual program dimming value */
  {
    TEMP_BYTE  = PROG_DIM_STEP;                   /* Read actual program dimming value */
    TEMP_BUFFER = DIM_PROGRAM[TEMP_BYTE];
    return (TEMP_BUFFER);                         /* Return message buffer */ 
  }
  
  if (REPLACE_TARGET == "MAINTEXT6")              /* Actual program speed */
  {
    TEMP_BYTE  = PROG_SPEED;                      /* Read actual program dimming value */
    TEMP_BUFFER = SPEED_PROGRAM[TEMP_BYTE];
    return (TEMP_BUFFER);                         /* Return message buffer */ 
  }

  if (REPLACE_TARGET == "MAINTEXT7")              /* Actual SSID */
  {
    TEMP_BUFFER = WLAN_SSID;                      /* Copy string */
    return (TEMP_BUFFER);                         /* Return message buffer */
  }
  
  if (REPLACE_TARGET == "MAINTEXT8")              /* Actual WIFI IP address */
  {
    TEMP_BUFFER = WiFi.softAPIP().toString();     /* Save actual Wifi local IP address */
    return (TEMP_BUFFER);                         /* Return message buffer */
  }
  
  if (REPLACE_TARGET == "MAINTEXT9")              /* Number of clients conected */
  {
    /* Save actual number of clients connected */
    TEMP_BUFFER = WiFi.softAPgetStationNum();              
    return (TEMP_BUFFER);                         /* Return message buffer */
  }
  
  if (REPLACE_TARGET == "MAINTEXT10")             /* Actual SW version */
  {
    TEMP_BUFFER = SW_VERSION;                     /* Save actual software version */
    return (TEMP_BUFFER);                         /* Return message buffer */
  }   
}


/****************************************************************************************/
/* Subroutine to replace placeholder in website submenu 1 with calculated values        */
/****************************************************************************************/
String SUB_WEB_PROCESSOR_SUBM1 (const String& REPLACE_TARGET)
{
  /* Definitions for dimming of color */
  const char *FIX_DIM[] = { "100", "85", "70", "60", "50", "40", "30", "20", "10"};
  /* Definitions for fixed color */
  const char *FIX_COLOR[] = { "---", "White", "Warm white", "Yellow", "Orange", "Red", "Dark purple", "Purple", "Bright purple", "Dark blue", "Blue", "Cyan", "Bright green", "Green"};
  byte TEMP_BYTE;                                 /* Temporary byte to determine right position */

  if (REPLACE_TARGET == "SUB1SELECT1")            /* Actual color */
  {
    /* Color selected */
    if ((ACT_PROGRAM >= 1) && (ACT_PROGRAM <= 13))
    {
      RECEIVE_BUFFER = "";                        /* ´Delete replacement string */
      /* Generate replacement string in loop for all options of watering mode */
      for(LOOP_COUNTER=1; LOOP_COUNTER <= 13; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (ACT_PROGRAM == LOOP_COUNTER)          /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += FIX_COLOR[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    else                                          /* Program or storage selected ? */
    {
      RECEIVE_BUFFER = "";                        /* ´Delete replacement string */
      /* Generate replacement string in loop for all options of watering mode */
      for(LOOP_COUNTER=0; LOOP_COUNTER <= 13; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (LOOP_COUNTER == 0)                    /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += FIX_COLOR[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  
  if (REPLACE_TARGET == "SUB1SELECT2")            /* Actual dim factor for color */
  {
    RECEIVE_BUFFER = "";                          /* ´Delete replacement string */
    /* Generate replacement string in loop for all options of dimming for color */
    for(LOOP_COUNTER=0; LOOP_COUNTER <= 8; LOOP_COUNTER++)
    {
      RECEIVE_BUFFER += "<option value=\"";
      RECEIVE_BUFFER += LOOP_COUNTER;
      RECEIVE_BUFFER += "\"";
      if (COLOR_DIM_STEP == LOOP_COUNTER)         /* Default selection ? */
      {
        RECEIVE_BUFFER += " selected>";
      }
      else                                        /* No default selection ? */
      {
        RECEIVE_BUFFER += ">";
      }
      RECEIVE_BUFFER += FIX_DIM[LOOP_COUNTER];
      RECEIVE_BUFFER += "</option><br>";
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
}


/****************************************************************************************/
/* Subroutine to replace placeholder in website submenu 2 with calculated values        */
/****************************************************************************************/
String SUB_WEB_PROCESSOR_SUBM2 (const String& REPLACE_TARGET)
{
  /* Definitions for programs */
  const char *PR_MODE[] = { "---", "Color change 1", "Color change 2", "Color change 3", "Color change 4", "Color change 5", "Steps", "Color flash 1", "Color flash 2", "Fireplace", "Thunderstorm"};   
  /* Definitions for dimming of color */
  const char *PR_DIM[] = { "100", "75", "50", "25"};
  /* Definitions for program speed */
  const char *PR_SPEED[] = { "100", "90", "80", "70", "60", "50", "40", "30", "20", "10"};
  
  byte TEMP_BYTE;                                 /* Temporary byte to determine right position */

  if (REPLACE_TARGET == "SUB2SELECT1")            /* Actual color */
  {
    /* Program selected */
    if ((ACT_PROGRAM >= 20) && (ACT_PROGRAM <= 29))   
    {
      TEMP_BYTE = ACT_PROGRAM - 19;               /* Calculate right offset */
      RECEIVE_BUFFER = "";                        /* ´Delete replacement string */
      /* Generate replacement string in loop for all options of watering mode */
      for(LOOP_COUNTER=1; LOOP_COUNTER <= 10; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (LOOP_COUNTER == TEMP_BYTE)            /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += PR_MODE[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    else                                          /* Single color or storage selected ? */
    {
      RECEIVE_BUFFER = "";                        /* ´Delete replacement string */
      /* Generate replacement string in loop for all options of watering mode */
      for(LOOP_COUNTER=0; LOOP_COUNTER <= 10; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (LOOP_COUNTER == 0)                    /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += PR_MODE[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
  
  if (REPLACE_TARGET == "SUB2SELECT2")            /* Actual dimming factor for program */
  {
    RECEIVE_BUFFER = "";                          /* ´Delete replacement string */
    /* Generate replacement string in loop for all options of program dimming */
    for(LOOP_COUNTER=0; LOOP_COUNTER <= 3; LOOP_COUNTER++)
    {
      RECEIVE_BUFFER += "<option value=\"";
      RECEIVE_BUFFER += LOOP_COUNTER;
      RECEIVE_BUFFER += "\"";
      if (LOOP_COUNTER == PROG_DIM_STEP)          /* Default selection ? */
      {
        RECEIVE_BUFFER += " selected>";
      }
      else                                        /* No default selection ? */
      {
        RECEIVE_BUFFER += ">";
      }
      RECEIVE_BUFFER += PR_DIM[LOOP_COUNTER];
      RECEIVE_BUFFER += "</option><br>";
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }

  if (REPLACE_TARGET == "SUB2SELECT3")            /* Actual speed factor for program */
  {
    RECEIVE_BUFFER = "";                          /* ´Delete replacement string */
    /* Generate replacement string in loop for all options of program speed */
    for(LOOP_COUNTER=0; LOOP_COUNTER <= 9; LOOP_COUNTER++)
    {
      RECEIVE_BUFFER += "<option value=\"";
      RECEIVE_BUFFER += LOOP_COUNTER;
      RECEIVE_BUFFER += "\"";
      if (LOOP_COUNTER == PROG_SPEED)             /* Default selection ? */
      {
        RECEIVE_BUFFER += " selected>";
      }
      else                                        /* No default selection ? */
      {
        RECEIVE_BUFFER += ">";
      }
      RECEIVE_BUFFER += PR_SPEED[LOOP_COUNTER];
      RECEIVE_BUFFER += "</option><br>";
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
}

/****************************************************************************************/
/* Subroutine to replace placeholder in website submenu 2 with calculated values        */
/****************************************************************************************/
String SUB_WEB_PROCESSOR_SUBM3 (const String& REPLACE_TARGET)
{
  /* Definitions for stores 1-5 */
  const char *STORE_MOD[] = { "---", "Storage 1", "Storage 2", "Storage 3", "Storage 4", "Storage 5"};
  /* Definitions for dimming of color */
  const char *STORE_DIM[] = { "100", "85", "70", "60", "50", "40", "30", "20", "10"};

  byte TEMP_BYTE;                                 /* Temporary byte to determine right position */

  if (REPLACE_TARGET == "SUB3SELECT1")            /* Actual store */
  {
    /* Store selected */
    if ((ACT_PROGRAM >= 15) && (ACT_PROGRAM <= 19))   
    {
      TEMP_BYTE = ACT_PROGRAM - 14;               /* Calculate right offset */
      RECEIVE_BUFFER = "";                        /* ´Delete replacement string */
      /* Generate replacement string in loop for all options of stores */
      for(LOOP_COUNTER=1; LOOP_COUNTER <= 5; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (LOOP_COUNTER == TEMP_BYTE)            /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += STORE_MOD[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    else                                          /* Single color or storage selected ? */
    {
      RECEIVE_BUFFER = "";                        /* ´Delete replacement string */
      /* Generate replacement string in loop for all options of stores */
      for(LOOP_COUNTER=0; LOOP_COUNTER <= 5; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (LOOP_COUNTER == 0)                    /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += STORE_MOD[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }

  if (REPLACE_TARGET == "SUB3VALUE1")             /* Red color value */
  {  
    if (SUB_MENU_FLAG == false)                   /* Display zero values */
    {
      RECEIVE_BUFFER = "0";                       /* Set receive buffer with 0 */
    } 
    else                                          /* Load actual valus from restore */
    {
      RECEIVE_BUFFER = STORE_RED_COLOR;           /* Copy actual storage red value into receive buffer */ 
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }

  if (REPLACE_TARGET == "SUB3VALUE2")             /* Green color value */
  {  
    if (SUB_MENU_FLAG == false)                   /* Display zero values */
    {    
      RECEIVE_BUFFER = "0";                       /* Set receive buffer with 0 */
    }
    else                                          /* Load actual valus from restore */
    {
      RECEIVE_BUFFER = STORE_GREEN_COLOR;         /* Copy actual storage green value into receive buffer */ 
    }   
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  } 
  
  if (REPLACE_TARGET == "SUB3VALUE3")             /* Blue color value */
  {  
    if (SUB_MENU_FLAG == false)                   /* Display zero values */
    {    
      RECEIVE_BUFFER = "0";                       /* Set receive buffer with 0 */
    }
    else                                          /* Load actual valus from restore */
    {
      RECEIVE_BUFFER = STORE_BLUE_COLOR;          /* Copy actual storage blue value into receive buffer */ 
    }   
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  } 
      

  if (REPLACE_TARGET == "SUB3SELECT2")            /* Actual dim factor for storage */
  {
    RECEIVE_BUFFER = "";                          /* ´Delete replacement string */
    if (SUB_MENU_FLAG == false)                   /* Display default 0 values */
    {    
      for(LOOP_COUNTER=0; LOOP_COUNTER <= 8; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (LOOP_COUNTER == 0)                    /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += STORE_DIM[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    else                                          /* Load actual valus from restore */
    {  
      /* Generate replacement string in loop for all options of dimming for color */
      for(LOOP_COUNTER=0; LOOP_COUNTER <= 8; LOOP_COUNTER++)
      {
        RECEIVE_BUFFER += "<option value=\"";
        RECEIVE_BUFFER += LOOP_COUNTER;
        RECEIVE_BUFFER += "\"";
        if (LOOP_COUNTER == STORE_DIM_VALUE)      /* Default selection ? */
        {
          RECEIVE_BUFFER += " selected>";
        }
        else                                      /* No default selection ? */
        {
          RECEIVE_BUFFER += ">";
        }
        RECEIVE_BUFFER += STORE_DIM[LOOP_COUNTER];
        RECEIVE_BUFFER += "</option><br>";
      }
    }
    return (RECEIVE_BUFFER);                      /* Return message buffer */
  }
}
